!*****************************************************************************
!
!  File : ldatom_info.f90
!    by : Alan Tackett
!    on : 8/30/95
!   for : PAW Method
!
!  Contains routine to load the atomic species tpye and also a specific atom
!
!
!*****************************************************************************

Module ldatom_info

Use atom_data
Use exchange_corr
Use paw_inout
Use word
Use misc
Use projectors
Use options_data
Use denvhat_pack
Use search_sort
Use vhartree_pack
Use crystal_data
Use mathlib
Use projectors
Use gausslib

Implicit NONE!!!

!*****************************************************************************
Contains
!*****************************************************************************

!***************************************
!
!  AtomAlloc - Allocates the MULTI-D arrays 
!
!***************************************

Subroutine AtomAlloc(WC, A)
  Type (Atom_Info_Fixed), Pointer :: A
  Type (Word_Context), Intent(INOUT) :: WC   

  Integer :: aerr

  If (Atomic_Mode == SIM_AE) then
     If ((A%Basis_Size == -1) .OR. (A%LCAO_Size == -1)) then
        Write(Error_Unit, *) "atomalloc: Can't Allocate arrays! Dim's not set!"
        Write(Error_Unit, *) "atomalloc: Atom Name : ", Trim(A%Atom_Name)
        Write(Error_Unit, *) "atomalloc: Basis_Size = ", A%Basis_Size
        Write(Error_Unit, *) "atomalloc: LCAO_Size  = ", A%LCAO_Size
     
        Call Word_GetAndPrint(WC, Error_Unit, 'atomalloc: ')
        STOP
     End If
  else 
     If ((A%Basis_Size == -1) .OR. (A%Mesh_Size == -1) .OR.   &
          (A%LCAO_Size == -1)) then
        Write(Error_Unit, *) "atomalloc: Can't Allocate arrays! Dim's not set!"
        Write(Error_Unit, *) "atomalloc: Atom Name : ", Trim(A%Atom_Name)
        Write(Error_Unit, *) "atomalloc: Basis_Size = ", A%Basis_Size
        Write(Error_Unit, *) "atomalloc: Mesh_Size  = ", A%Mesh_Size
        Write(Error_Unit, *) "atomalloc: LCAO_Size  = ", A%LCAO_Size
     
        Call Word_GetAndPrint(WC, Error_Unit, 'atomalloc: ')
        STOP
     End If
  End If

  if (Print_Level == PRINT_VERBOSE) then
     write(Log_Unit, *) "#atomalloc : Allocating arrays for phi, tphi, tp"
  End If
    
  Allocate(A%Phi(A%Mesh_Size, A%Basis_Size), STAT=aerr)
  Call Check_Error(aerr, 'atomalloc: Error Allocating PHI!', Error_Unit, &
                   .TRUE., WC, 'atomalloc: ')

  Allocate(A%TPhi(A%Mesh_Size, A%Basis_Size), &
           A%TPhi_LCAO(A%LCAO_Size, A%Basis_Size), STAT=aerr)
  Call Check_Error(aerr, 'atomalloc: Error Allocating TPHI!', Error_Unit, &
                   .TRUE., WC, 'atomalloc: ')

  !Allocate(A%TP(A%Mesh_Size, A%Basis_Size), &
  !     A%TP_Deriv(A%Mesh_Size, A%Basis_Size), STAT=aerr)
  Allocate(A%TP(A%Mesh_Size, A%Basis_Size),  STAT=aerr)
  Call Check_Error(aerr, 'atomalloc: Error Allocating TP!', Error_Unit, &
                   .TRUE., WC, 'atomalloc: ')

  Return
End Subroutine

!**********************************************************************
!
! FillArray - Loads a 1-D array of data
!
!**********************************************************************

Subroutine FillArray(WC, A, A_Size, IDText)
  Type (Word_Context), Intent(INOUT) :: WC   
  Real,    Intent(OUT) :: A(:)
  Integer, Intent(IN)  :: A_Size
  character*(*)        :: IDText

  Character*50 :: str, token
  Integer      :: ierr, tlen

  str = "FillArray(" // Trim(IDText)   


!***write(*,*) trim(str), 'STart---------------- N=',a_Size

  Call GetNumbers(WC, A, A_Size)
  if (W_ERROR /= W_OK) Then
     Write(Error_Unit, *) trim(str)," data): Array_Size=",A_Size
     Call EOF_Error(WC, trim(str) // " data):");     
  End If

  Call GetNextWord(WC, token, tlen)
  if (W_ERROR /= W_OK) Call EOF_Error(WC, trim(str) // " END)");

  Call UpperCase(token)
  if (token /= "END") then
     Write(Error_Unit,*)"ldatom: END Not found while loading basis "
     Write(Error_Unit,*) "ldatom: Ignoring ALL text until END is found!"
     Call Word_GetAndPrint(WC, Error_Unit, "LdAtomType: ")

     Do While (token /= "END")
        Call GetNextWord(WC, token, tlen)
        if (W_ERROR /= W_OK) Call EOF_Error(WC, trim(str) // " scan)");
        Call UpperCase(token)
     End Do
  End IF

  Return
End Subroutine

!**********************************************************************
!
!  LoadArray - Allocates and Loads a 1-D array of data
!
!**********************************************************************

Subroutine LoadArray(WC, A, A_Size, IDText)
  Type (Word_Context), Intent(INOUT) :: WC   
  Real,    Pointer    :: A(:)
  Integer, Intent(IN) :: A_Size
  character*(*)       :: IDText

  Character*50 :: str, token
  Integer      :: ierr, tlen

  str = "LoadArray(" // Trim(IDText)

  if (Print_Level == PRINT_VERBOSE) then
     Write(Log_Unit, *) Trim(Str), ') : Loading with size =',A_Size
  End IF

  if (A_Size <= 0) then
     str = trim(str) // " alloc):"
     Write(Error_Unit,*) trim(str), "Can't load matrix no Size!"
     Call Word_GetAndPrint(WC, Error_Unit, trim(str))
     STOP
  End If
  
  Allocate(A(A_Size), STAT=ierr)
  Call Check_Error(ierr, trim(str) // ' Error during allocation!',  &
                   Error_Unit, .TRUE., WC, trim(str))

  Call FillArray(WC, A, A_Size, IDText)

  Return
End Subroutine

!**********************************************************************
!
!  LoadDenVhat - Loads the Density and V_hat arrays
!
!**********************************************************************

Subroutine LoadDenVhat(WC, A, LUT, A_Size, IDtext)
  Type (Word_Context), Intent(INOUT) :: WC   
  Real,    Pointer :: A(:)
  Integer, Pointer :: LUT(:)
  Integer, Intent(IN) :: A_Size
  Character*(*)       :: IDText

  Logical       :: FirstTime, Exact
  Character*100 :: str, token
  Integer       :: i, j, k, ierr, tlen
  Integer       :: N, Ind(3)  
  Real          :: v

  FirstTime = .FALSE.
  str = "LoadDenVhat(" // Trim(IDtext)

  If (A_Size <= 0) then
     str = Trim(str) // " alloc):"
     Write(Error_Unit,*) trim(str), "Can't load matrix no Size!"
     Call Word_GetAndPrint(WC, Error_Unit, trim(str))
     STOP
  End If

!Write(Log_Unit, *) 'lddenvhat: A_Size=',A_size, ' * Associated(A)=',Associated(A)    
  if (.NOT. Associated(A)) then
     Allocate(A(A_Size), STAT=ierr)
     Call Check_Error(ierr, trim(str) // ' Error during allocation!',  &
                      Error_Unit, .TRUE., WC, trim(str) // " alloc):")

     if (.NOT. Associated(LUT)) then
        FirstTime = .TRUE.
       Allocate(LUT(A_Size), STAT=ierr)
       Call Check_Error(ierr,trim(str)//' Error during LUT_DenVhat alloc!', &
                  Error_Unit, .TRUE., WC, trim(str) // ": LUT_denvhat Alloc):")
     End If
  End If

  Do i = 1, A_Size
!Write(Log_Unit,*) 'lddenvhat: i=',i

    Call GetNumbers(WC, Ind)     !** Get the indices
    Call GetNumbers(WC, V)       !** and the corresponding value

    j = DenVhat_Encode(Ind(1), Ind(2), Ind(3))

    if (FirstTime) then       !** First time then store values only
      A(i) = V
      LUT(i) = j
    else                      !** Check for index match in a previous run
      k = Linear_Search(LUT, j, Exact)
!Write(Log_Unit,*) 'lddenvhat: Match = ',k, ' * Decode=',j
      if (Exact) then         !** Got an Exact Match with a preceeding call
        A(k) = V                    !** So store the value in the right spot
      else                    !** Error!!
        Write(Error_Unit, *) 'LoadDenVhat: No match was found!'
        Write(Error_Unit, *) 'LoadDenVhat:  (n1l1, n2l2, L) : ', Ind
        Call Word_GetAndPrint(WC, Error_Unit, 'LoadDenVhat:')
        STOP
      End If
    End If
  End Do

  Call GetNextWord(WC, token, tlen)  !** Should Be and END Statement
  if (token /= "END") then
     Write(Error_Unit,*)"ldatom: END Not found while loading basis "
     Write(Error_Unit,*) "ldatom: Ignoring ALL text until END is found!"
     Call Word_GetAndPrint(WC, Error_Unit, "ldatom: ")

     Do While (token /= "END")
        Call GetNextWord(WC, token, tlen)
        if (W_ERROR /= W_OK) Call EOF_Error(WC, trim(str) // " scan)");
        Call UpperCase(token)
     End Do
  End IF

  Return
End Subroutine


!**********************************************************************
!
!  LoadVHartree - Loads the VHartree array
!
!**********************************************************************

Subroutine LoadVhartree(WC, A, LUT, A_Size)
  Type (Word_Context), Intent(INOUT) :: WC   
  Real,    Pointer :: A(:)
  Integer, Pointer :: LUT(:)
  Integer, Intent(IN) :: A_Size

  Character*100 :: str, token
  Integer       :: i, j, k, ierr, tlen
  Integer       :: N, Ind(5)  
  Real          :: v

  str = "LoadVHartree :"

       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading VHartree"
       End IF                
  If (A_Size <= 0) then
     str = Trim(str) // " alloc):"
     Write(Error_Unit,*) trim(str), "Can't load matrix no Size!"
     Call Word_GetAndPrint(WC, Error_Unit, trim(str))
     STOP
  End If
    
  Allocate(A(A_Size), STAT=ierr)
  Call Check_Error(ierr, trim(str) // ' Error during allocation!',  &
       Error_Unit, .TRUE., WC, trim(str) // " alloc):")

  if (.NOT. Associated(LUT)) then
     Allocate(LUT(A_Size), STAT=ierr)
     Call Check_Error(ierr,trim(str)//' Error during LUT_VHartree alloc!', &
               Error_Unit, .TRUE., WC, trim(str) // ": LUT_VHartreee Alloc):")
  End If

  Do i = 1, A_Size
    Call GetNumbers(WC, Ind)     !** Get the indices
    Call GetNumbers(WC, V)       !** and the corresponding value

    LUT(i) = VHartree_Encode(Ind(1), Ind(2), Ind(3), Ind(4), Ind(5))
    A(i) = V
  End Do

  Call GetNextWord(WC, token, tlen)  !** Should Be and END Statement
  if (token /= "END") then
     Write(Error_Unit,*)"ldatom: END Not found while loading basis "
     Write(Error_Unit,*) "ldatom: Ignoring ALL text until END is found!"
     Call Word_GetAndPrint(WC, Error_Unit, "ldatom: ")

     Do While (token /= "END")
        Call GetNextWord(WC, token, tlen)
        if (W_ERROR /= W_OK) Call EOF_Error(WC, trim(str) // " scan)");
        Call UpperCase(token)
     End Do
  End IF

  Return
End Subroutine

!**********************************************************************
!
!  Check_Atom - Does a quick test to make sure I got all the data
!
!**********************************************************************

Subroutine Check_Atom(WC, A)
  Type (Word_Context), Intent(INOUT) :: WC   
  Type(Atom_Info_Fixed), Pointer :: A
  Logical                        :: Ok

  Ok = .TRUE.

  if (A%Basis_Size < 1) then  
     Write(Error_Unit, *) 'ldatomType: Error Basis_Size =',A%Basis_size
     Ok = .FALSE.
  End If

  if (A%LCAO_Size < 1) then  
     Write(Error_Unit, *) 'ldatomType: Error LCAO_Size =',A%LCAO_size
     Ok = .FALSE.
  End If

  if (A%LCAO_Step <= 0) then  
     Write(Error_Unit, *) 'ldatomType: Error LCAO_Step =',A%LCAO_Step
     Ok = .FALSE.
  End If

  If (Atomic_Mode == SIM_PAW) then
     if (A%Mesh_Size < 1) then  
        Write(Error_Unit, *) 'ldatomType: Error Mesh_Size = ', A%Mesh_size
        Ok = .FALSE.
     End If

     if (A%Overlap_Size < 1) then  
        Write(Error_Unit,*) 'ldatomType: Error Overlap_Size = ', A%Overlap_size
        Ok = .FALSE.
     End If

     if (A%Hartree_Size < 1) then  
        Write(Error_Unit,*) 'ldatomType: Error Hartree_Size = ', A%Hartree_Size
        Ok = .FALSE.
     End If

     if (A%DenVhat_Size < 1) then  
        Write(Error_Unit,*) 'ldatomType: Error DenVhat_Size = ', A%DenVhat_Size
        Ok = .FALSE.
     End If

     if (A%Hartree_Size < 1) then  
        Write(Error_Unit,*) 'ldatomType: Error Hartree_Size = ', A%Hartree_Size
        Ok = .FALSE.
     End If

     if (A%Mesh_Step <= 0) then  
        Write(Error_Unit, *) 'ldatomType: Error Mesh_step = ',  A%Mesh_Step
        Ok = .FALSE.
     End If


     if (A%Rc <= 0) then  
        Write(Error_Unit, *) 'ldatomType: Error Rc = ',  A%Rc
        Ok = .FALSE.
     End If

     if (A%Rbox <= 0) then  
        Write(Error_Unit, *) 'ldatomType: Error Rbox = ',  A%Rbox
        Ok = .FALSE.
     End If
  End If

  If (.NOT. Ok) then
     Call Word_GetAndPrint(WC, Error_Unit, 'Check_Atom:')
     STOP
  End IF

  Return
End Subroutine


!****************************************************************************
!
!  ldatomtype - Procedure to load all the atom specific information that 
!           is help fixed throughout the calculation.
!
!      NOTE : This file also contains a support routine for allocation
!
!  Format of AtomType in input file is shown below.  All keywords are shown
!  in uppercase. Curly braces denote an optional parameter.
!
!  NOTE : NO checking is done to ensure that ALL data is loaded!!!
!
!    MAX_ATOMTYPES  MaxAtomType           # Previously defined
!
!    ATOMTYPE  AtomName(string)
!      ATOMXCTYPE    character
!      BASIS_SIZE    integer
!      MESH_SIZE     integer
!      DENVHAT_SIZE       integer
!      HARTREE_SIZE  integer
!      ATOMIC_CHARGE real
!      CORE_CHARGE   real
!      SHAPE_TYPE    character  
!      RC            real {unit}    # Cutoff Radius   
!      MESH_STEP     real {unit}
!      ENERGY        real {unit}
!      LCAO_SIZE     integer        # Size of tphi_LCAO functions
!      LCAO_STEP     real {unit}    # Radial Step size of tphi_LCAO functions
!
!      ENHANCE_Rc    real {unit}    # Local Enhancement radius > Rc
!
!      TPHI_LCAO Basis_Index(integer)   # One for each TPhi basis function
!          Data listed in ascending order
!      END
!
!      ORBITALS
!          Orbital_1(integer)
!               ...
!          Orbital_BASIS_SIZE(integer)
!      END
!
!      PHI Basis_Index(Integer)         # One for each Basis function
!         Data Listed in ascending order 
!      END
!
!      TPHI Basis_Index(Integer)        # One for each |phi~>
!         Data Listed in ascending order
!      END
!
!      TPROJECTOR Basis_Index(Integer)   # one for each |p~>
!         Data listed in ascending order
!      END
!
!      CORE_DENSITY
!         Data Listed in ascending order
!      END
!
!      OVERLAP_MATRIX                   # Size not needed because of definition
!         Data Listed in ascending order
!      END
!
!      KINETIC_ENERGY_MATRIX
!         Data Listed in ascending order
!      END
!
!      V_ION_MATRIX
!         Data Listed in ascending order
!      END
!
!      DENSITY
!        n1l1 n2l2 L   value
!         ..   ..  ..   ...
!      END
!
!      V_HAT
!        n1l1 n2l2 L   value
!         ..   ..  ..   ...
!      END
!
!      V_HARTREE
!        n1l1 n2l2 n3l3 n4l4 L   value
!         ..   ..   ..   ..  .    ...
!      END
!    END                  # End of Atom Definition
!
!**********************************************************************

Subroutine LdAtomType(WC)
  Type (Word_Context), Intent(INOUT) :: WC   

  Character*100 :: Token,Token2
  Logical      :: NewToken, Ok
  Integer      :: i, j, k, L, m, tlen, intnum, base_i, base_j
  Real         :: realnum, scale, den, Vxc, Exc, gmag ,V_local
  Integer, Pointer :: LUT(:)
  Integer, Pointer :: IntTemp(:)
  Real, Pointer    :: RealTemp(:,:), Rtmp(:)
  Logical          :: AllocatedBigArrays
  Type(Atom_Info_Fixed), Pointer :: A

  If (Atomic_Mode /= SIM_PAW) then
     Write(Log_Unit,*) 'LdAtomtype: Attempting to load a PAW', &
          ' pseudopotential when NOT in PAW mode!!!!!'
     Call Word_GetAndPrint(WC, Error_Unit, 'ldAtomType: ')
     STOP
  End If

  if (Atom_types >= Max_Atom_Types) then
     write(Error_Unit, *) 'ldatom: Error Loading atom type!'
     write(Error_Unit, *) 'ldatom: Not Enough space allocated!'
     write(Error_Unit, *) 'ldatom: Max_Atom_Types = ', Max_Atom_Types

     Call Word_GetAndPrint(WC, Error_Unit, 'ldatomtype: ')
     STOP
  End If

  AllocatedBigArrays = .FALSE.
  Atom_types = Atom_Types + 1
  A => AtomType_Info(Atom_types)

  Write(Log_Unit,*) 'LdAtomType: TypeIndex = ', Atom_Types

        !***** Initialize Constants to -1 or 0 for error checking *****
  A%Mesh_Size     = -1;     A%Basis_Size    = -1  
  A%CoreTail_Points = -1
  A%Overlap_Size  = -1;     A%Mass = 1
  A%Hartree_Size  = -1;     A%Atomic_Charge = 0
  A%Core_Charge   = 0;      A%Orbitals_Size = 0
  A%Mesh_Step     = -1;     A%Rc_RS_Scale = 1.5
  A%Atom_Energy   = 0;      A%DenVhat_Size  = -1
  A%Rc            = -1;     A%Rbox          =  1  !** Rbox Not used anymore
  A%LCAO_Step     = -1;     A%LCAO_Size     = -1
  A%Gcut_Proj = 0
  A%CoreTail_SelfEnergy = 0;A%CoreTail_HatEnergy = 0
  !A%V_local       = 0;      A%Gcut_Proj = 0
  V_local=1.e20

  Nullify(A%Density);    Nullify(A%LUT_DenVhat)
  Nullify(A%V_Hartree);  Nullify(A%LUT_V_Hartree)
  Nullify(A%Valence_Orbitals);  Nullify(A%Init_Occ)
  Nullify(A%L_Value);  Nullify(A%V_ion);  Nullify(A%V_hat);

  Call GetNextWord(WC, A%Atom_Name, Tlen)

  If (Print_Level <= PRINT_COMMANDS) then
     Write(Log_Unit, *) ">AtomType ", Trim(A%Atom_Name)
  End IF

  i = SearchAddList(A%Atom_Name, AtomType_LUT, NewToken)
  if (.NOT. NewToken) then     !** Name previously used
     Write(Error_Unit, *) "LdAtomType: Name previously used! Name =", &
                           Trim(A%Atom_Name), " * Index =", i
     Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomType:')
     STOP
  End If
   

  Call GetNextWord(WC, token, Tlen)
  Call UpperCase(Token)
!*** write(*,*) 'ldatomtype: Start loop tlen=',tlen, ' * Token=',Trim(Token)  

  Do While (Trim(token) /= "END")
    NewToken = .TRUE.

    if (Trim(token) == "BASIS_SIZE") then
       Call GetNumbers(WC, A%Basis_Size)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Basis_Size = ", A%Basis_Size
       End IF
    else if (Trim(token) == "MESH_SIZE") then
       Call GetNumbers(WC, A%Mesh_Size) 
       if(A%CoreTail_Points<1) A%CoreTail_Points=A%Mesh_Size
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Mesh_Size = ", A%Mesh_Size
       End IF
    else if (Trim(token) == "CORETAIL_POINTS") then
       Call GetNumbers(WC, A%CoreTail_Points) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    CoreTail_Points = ", A%CoreTail_Points
       End IF
    else if (Trim(token) == "DENVHAT_SIZE") then
       Call GetNumbers(WC, A%DenVHat_Size) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    DenVHat_Size = ", A%DenVHat_Size
       End IF
    else if (Trim(token) == "OVERLAP_SIZE") then
       Call GetNumbers(WC, A%Overlap_Size) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Overlap_Size = ", A%Overlap_Size
       End IF
    else if (Trim(token) == "LCAO_SIZE") then
       Call GetNumbers(WC, A%LCAO_Size) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    LCAO_Size = ", A%LCAO_Size
       End IF
    else if (Trim(token) == "LCAO_STEP") then
       NewToken = .NOT. GetRealsWithUnit(WC, A%LCAO_Step, Length_Unit, Token)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    LCAO_Step = ", A%LCAO_Step
       End IF
    else if (Trim(token) == "HARTREE_SIZE") then
       Call GetNumbers(WC, A%HARTREE_Size)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Hartree_Size = ", A%Hartree_Size
       End IF  
    else if (Trim(token) == "ATOMIC_CHARGE") then
       Call GetNumbers(WC, A%Atomic_Charge)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Atomic_Charge = ", A%Atomic_Charge
       End IF
    else if (Trim(token) == "ATOMXCTYPE") then
       Call GetNextWord(WC, token2, Tlen)
       Call UpperCase(Token2)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:   XC_TYPE = ", Token2 
       End IF
       i=0
       If (TRIM(Token2) == 'LDA-PW') i=XC_LDA_PW
       If (TRIM(Token2) == 'GGA-PBE') i=XC_GGA_PBE
       If (XC_TYPE_SET) then       ! Check that this atomdata file is compatible
                                   !  with previously set XC type
          If (i /= XC_TYPE) then
              Write(Error_Unit,*) 'Error in XC_Type --', Token2,&
                'Is not compatible with previously set XC_Type ',XC_TYPE
              Write(Log_Unit,*) 'Error in XC_Type --', Token2,&
                'Is not compatible with previously set XC_Type ',XC_TYPE
             Stop
          EndIf

       Else

         XC_TYPE_SET=.true.
         XC_TYPE=i
       EndIf
    else if (Trim(token) == "VLOC") then
       Call GetNumbers(WC, V_Local)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    V_Local = ", V_Local
       End IF
    Else if (Trim(token) == "CORE_CHARGE") then
       CAll GetNumbers(WC, A%Core_Charge)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Core_Charge = ", A%Core_Charge
       End IF 
    else if (Trim(token) == "SHAPE_TYPE") then
         Call GetNextWord(WC, A%Shape_type, Tlen)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Shape_type = ", A%Shape_type
       End IF                
    else if (Trim(token) == "RC") then
       NewToken = .NOT. GetRealsWithUnit(WC, A%Rc, Length_Unit, Token)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Rc = ", A%Rc
       End IF                
    else if (Trim(token) == "ENHANCE_RC") then
       NewToken = .NOT. GetRealsWithUnit(WC, A%Rbox, Length_Unit, Token)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Rbox = ", A%Rbox
       End IF                
    else if (Trim(token) == "MESH_STEP") then
       NewToken = .NOT. GetRealsWithUnit(WC, A%Mesh_Step, Length_Unit, Token)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Mesh_Step = ", A%Mesh_Step
       End IF
    else if (Trim(token) == "ENERGY") then
       NewToken = .NOT.  GetRealsWithUnit(WC,A%Atom_Energy,Energy_Unit, Token)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Energy = ", A%Atom_energy
       End IF
    else if (Trim(token) == "ORBITALS") then
       Allocate(A%L_Value(A%Basis_Size), STAT=j)
       Call Check_Error(j, 'ldatom(orbitals data) Error during allocation!',  &
                   Error_Unit, .TRUE., WC, "ldatom(orbitals data): ");     
       Call LoadArray(WC, RTmp, A%Basis_Size, "orbitals")
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(orbitals data): ");     
       A%L_Value = RTmp
       DeAllocate(Rtmp)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Orbitals = ", A%L_Value
       End IF                
    else if (Trim(token) == "PHI") then
       Call GetNumbers(WC, j)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(phi index): ");     
       if (.NOT. AllocatedBigArrays) Call AtomAlloc(WC, A)
       AllocatedBigArrays = .TRUE.
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading Phi = ", j
       End IF                
       Call FillArray(WC, A%Phi(:,j), A%Mesh_Size, "phi")
    else if (Trim(token) == "TPHI") then
       Call GetNumbers(WC, j)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(tphi index): ");     
       if (.NOT. AllocatedBigArrays) Call AtomAlloc(WC, A)
       AllocatedBigArrays = .TRUE.
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading TPhi = ", j
       End IF                
       Call FillArray(WC, A%TPhi(:,j), A%Mesh_Size, "tphi")
    else if (Trim(token) == "TPHI_LCAO") then
       Call GetNumbers(WC, j)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(tphi_LCAO index): ")  
       if (.NOT. AllocatedBigArrays) Call AtomAlloc(WC, A)
       AllocatedBigArrays = .TRUE.
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading TPhi_LCAO = ", j
       End IF                
       Call FillArray(WC, A%TPhi_LCAO(:,j), A%LCAO_Size, "tphi_LCAO")
    else if (Trim(token) == "TPROJECTOR") then
       Call GetNumbers(WC, j)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(tproj index): ");     
       if (.NOT. AllocatedBigArrays) Call AtomAlloc(WC, A)
       AllocatedBigArrays = .TRUE.
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading TP = ", j
       End IF                
       Call FillArray(WC, A%TP(:,j), A%Mesh_Size, "tproj")    
    ! TP_Deriv no longer used
    !else if (Trim(token) == "DERIVTPROJECTOR") then
    !   Call GetNumbers(WC, j)
    !   if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(tproj_deriv index): ");
    !   if (.NOT. AllocatedBigArrays) Call AtomAlloc(WC, A)
    !   AllocatedBigArrays = .TRUE.
    !   if (Print_Level == PRINT_VERBOSE) then
    !      Write(Log_Unit, *) "ldatomtype:    Loading TP_Deriv = ", j
    !   End IF                
    !   Call FillArray(WC, A%TP_Deriv(:,j), A%Mesh_Size, "tproj_deriv")    
    else if (Trim(token) == "SHAPE_FUNC") then
       Call LoadArray(WC, A%Shape_Func, A%Mesh_Size, "shape_func")
    else if (Trim(token) == "VLOCFUN") then
       Call LoadArray(WC, A%RadR_Vlocal, A%Mesh_Size, "vloc_fun")
    else if (Trim(token) == "CORE_DENSITY") then
       Call LoadArray(WC, A%Core_Density, A%Mesh_Size, "core_density")
    else if (Trim(token) == "CORETAIL_DENSITY") then
       Call LoadArray(WC, A%CoreTail_Density, A%CoreTail_Points, "coretail_density")
    else if (Trim(token) == "OVERLAP_MATRIX") then
       Call LoadArray(WC, A%List_Oij, A%overlap_size, "overlap")    
    else if (Trim(token) == "KINETIC_ENERGY_MATRIX") then
       Call LoadArray(WC, A%Kinetic, A%overlap_size, "kinetic_energy")    
    else if (Trim(token) == "V_ION_MATRIX") then
       Call LoadArray(WC, A%V_ion, A%overlap_size, "v_ion")    
    else if (Trim(token) == "DENSITY") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading Density"
       End IF                
       Call LoadDenVhat(WC,A%Density, A%LUT_DenVhat, A%DenVhat_Size, "Density")
    else if (Trim(token) == "V_HAT") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Loading V_Hat"
       End IF                
       Call LoadDenVhat(WC, A%V_hat, A%LUT_DenVhat, A%DenVhat_Size, "V_Hat")
    else if (Trim(token) == "V_HARTREE") then
       Call LoadVHartree(WC, A%V_Hartree, A%LUT_V_Hartree, A%Hartree_Size)
    else if (Trim(token) == "CORETAILSELFENERGY") then
       Call GetNumber(WC, A%CoreTail_Selfenergy)
       Write(Log_Unit,*) 'ldatomtype:  CoreTail Self-Energy:',A%CoreTail_Selfenergy
       Call GetNextWord(WC, token, Tlen)
    else if (Trim(token) == "CORETAILHATENERGY") then
       Call GetNumber(WC, A%CoreTail_Hatenergy)
       Write(Log_Unit,*) 'ldatomtype:  CoreTail Hat-Energy:',A%CoreTail_Hatenergy
       Call GetNextWord(WC, token, Tlen)
    else if (Trim(token) == "HAT_SELF-ENERGY") then
       Call GetNumber(WC, i)
       Allocate(A%Hat_SelfEnergy(0:i))
       Do j=0, i
          Call GetNumber(WC, L)
          Call GetNumber(WC, A%Hat_SelfEnergy(L))
       End Do
       Call GetNextWord(WC, token, Tlen)
       Write(Log_Unit,*) 'ldatomtype:    Hat Self-Energy:',A%Hat_SelfEnergy
    else if (Trim(Token) == "MASS") then
       Call GetNumber(WC, A%Mass)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatomtype:  Mass = ", A%Mass
       End IF 
    else 
       Write(Error_Unit, *) "ldAtomType: Unknown Option: ", Trim(Token)
    End If    

    If (NewToken) then
       Call GetNextWord(WC, token, tlen)
       Call UpperCase(Token)
!*** write(*,*) 'ldatomtype: tlen=',tlen, ' * Token=',Trim(Token)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(next word): ")
    End If
  End Do

  If (ABS(V_local)<1.e10) then
        If (.not.Associated(A%RadR_Vlocal)) then 
          Allocate(A%RadR_Vlocal(A%Mesh_Size), stat=i)
           if (i/=0) then
             Call Check_Error(i, 'ldatomtype: Error Allocating A%RadR_Vlocal', &
             Error_Unit, .TRUE., WC, 'LdAtomType: ')
             stop
           endif
        Endif
        A%RadR_Vlocal=V_local*A%Shape_Func
  Endif


  Call Check_Atom(WC, A)      !** Do a cursory check of the atom **
   
    !**** Create nlm LUT and nl base arrays if needed ****
  IF (Atomic_Mode == SIM_PAW) then
     k = 0
     Do i=1, A%Basis_Size
        k = k + 2*A%L_Value(i) + 1
     End Do

     A%nlm_Size = K
     Allocate(A%nl_Base(A%Basis_Size), A%nlm_LUT(3,A%nlm_Size),  &
          A%Oij(k, k), A%Core_Xchange(A%MEsh_Size), STAT=i)
     Call Check_Error(i, 'ldatomtype: Error Allocating nlm_base and nlm_LUT!',&
          Error_Unit, .TRUE., WC, 'LdAtomType: ')

     k = 0
     Do i=1, A%Basis_Size
        L = A%L_Value(i)
        A%nl_base(i) = k + 1
     
        Do M = -L, L
           k = k + 1
           A%nlm_LUT(:,k) = (/i,L,M/)
        End Do
     End Do

     A%Oij = 0
     k = 0
     Do i=1, A%Basis_Size
        Do j=1, i
           If (A%L_Value(i) == A%L_Value(j)) then
              k = k + 1
              
              L = A%L_Value(i)           
              base_i = A%nl_base(i)+L
              base_j = A%nl_base(j)+L

              Do M=-L, L
                 A%Oij(base_i+M, Base_j+M) = A%List_Oij(k)
                 A%Oij(base_j+M, Base_i+M) = A%List_Oij(k)
              End Do
           End If
        End Do
     End Do

     !*** Make the Coulomb Matrix aQlm, and aVlm ****

      !*** First sort the V_hartree LUT in ascending order ***
     Allocate(LUT(A%Hartree_Size),RTmp(A%Hartree_Size), &
          IntTEmp(A%Hartree_Size), STAT=i)
     Call Check_Error(i, 'ldatomtype: Error Allocating Temp LUT!',&
          Error_Unit, .TRUE., WC, 'LdAtomType: ')
  
     CAll Insertion_Sort(A%LUT_V_Hartree, LUT, .TRUE.)

     IntTemp = A%LUT_V_Hartree(LUT)
     RTmp = A%V_Hartree(LUT)
     A%LUT_V_Hartree = IntTemp
     A%V_Hartree = RTmp

     A%Cijkl_Size = -1; A%QVlm_Size = -1
     Call Coulomb_Matrix(A, A%Cijkl_Size)     !** First determine the size
     Call Orbital_Matrix(A, A%QVlm_Size) 

     Allocate(A%LUT_Cijkl(A%Cijkl_Size), A%Cijkl(A%Cijkl_Size), &
          A%aQlm(A%QVlm_Size), A%aVlm(A%QVlm_Size), A%LUT_Orb(A%QVlm_Size), &
          STAT=i)
     Call Check_Error(i, 'ldatomtype: Error Allocating Coulomb and orbital matrices',&
          Error_Unit, .TRUE., WC, 'LdAtomType: ')

!Write(LOG_Unit,*) 'ldatomtype: Cijkl_Size=',A%Cijkl_Size
!Write(LOG_Unit,*) 'ldatomtype: QVLM_Size=',A%QVLM_Size
     Call Coulomb_Matrix(A, A%Cijkl_Size)     !** Actually Store the matrices
     Call Orbital_Matrix(A, A%QVlm_Size) 
!write(Log_Unit,*) 'LdAtomType: aQlm=',A%aQlm

!OPEN(29,FILE="proj.out", RECL=10000)
!Do i=1, A%Mesh_Size
!  Write(29,*) i, (i-1)*A%Mesh_Step, A%TP(i,:)
!End DO
!CLOSE(29)

      !****** Scale the Core Density to that in the paper ******
     Do i=2, A%Mesh_Size  
        A%Core_Density(i) = A%Core_Density(i)/(Four_PI*(A%Mesh_Step*(i-1))**2)
     End Do
        !A%Core_Density(1)= 2*A%Core_Density(2)-A%Core_Density(3)
        A%Core_Density(1)= 3*(A%Core_Density(2)-A%Core_Density(3)) &
                                   +A%Core_Density(4)

   if (.NOT.Associated(A%CoreTail_Density)) then
      Allocate(A%CoreTail_Density(A%CoreTail_Points),stat=i)
          if (i /= 0) then
             write(Error_unit,*) 'ldatomtype:  Error in Coretail', A%CoreTail_Points,i
             stop
          endif
     A%CoreTail_Density=0
     A%Qeffion=-A%Atomic_Charge+A%Core_charge
   else
     Do i=2, A%CoreTail_Points  
        A%CoreTail_Density(i) = A%CoreTail_Density(i)/(Four_PI*(A%Mesh_Step*(i-1))**2)
     End Do
        A%CoreTail_Density(1)= 3*(A%CoreTail_Density(2)-A%CoreTail_Density(3)) &
                                   +A%CoreTail_Density(4)

     Allocate(Rtmp(A%Mesh_Size),stat=i)
          if (i /= 0) then
             write(Error_unit,*) 'ldatomtype:  Error in Rtmp', A%Mesh_Size,i
             stop
          endif
     Do i=1,A%Mesh_Size
        Rtmp(i)=(A%Core_Density(i)- &
            A%CoreTail_Density(i))*(Four_PI*(A%Mesh_Step*(i-1))**2)
     enddo
     A%Qeffion=-A%Atomic_Charge+IntSimpson(A%Mesh_Size,A%Mesh_Step,Rtmp)
     Deallocate(Rtmp)
    Endif
     write(Log_Unit,*) 'ldatomtype: Qeffion = ', A%Qeffion

! This part has been changed -- all functions f(r) are assumed to be used
!        as f(r)/r*Ylm
!!  !******Input projectors are divided by r**L -- multiply by r**l
!!   (note -- this will give A%TP=p(r)/r, where p(r) is function defined in PRB)
!!  Need to check this for TP_Deriv
!   write(6,*) 'Abasis',A%Basis_Size,A%L_value
!   do j=1,A%Basis_Size
!      If (A%L_value(j)>0) then
!         Do i=2, A%Mesh_Size  
!            A%TP(i,j) = A%TP(i,j)*(A%Mesh_Step*(i-1))**(A%L_Value(j) )
!            A%TP_Deriv(i,j)=A%TP_Deriv(i,j)*(A%Mesh_Step*(i-1))**(A%L_Value(j))
!         End Do
!         A%TP(1,j) = 0
!         A%TP_Deriv(1,j) = 0
!      Endif
!   End Do

!  !*** Remove the r factor from  TPhi_LCAO ***
!  A%TPhi_LCAO(1,:) = 0
!  A%TP_Deriv(1,:) = 0
!  Do i=2, A%LCAO_Size  
!     A%TPhi_LCAO(i,:) = A%TPhi_LCAO(i,:)/(A%Mesh_Step*(i-1))
!  End Do

!!      !***** print the Projectors out to a file ********
!!    Do i=1, A%Basis_Size
!!       Write(token, *) i
!!       token = TRIM(A%Atom_Name) // trim(ADJUSTL(token)) // '.txt'
!!       Open(SCRATCH_UNIT, FILE=token, RECL=10000)
!!
!!       Write(SCRATCH_UNIT, *) '1.Index   2.R    3.Phi    4.Phi~   5.P~', &
!!'   6.CoreDen/r^2   7.P~/r    8.Phi/r    9.Phi~/r '
!!!!       !     Write(SCRATCH_UNIT, *) ' 0      0     0      0     0'
!!       Do j=2, A%Mesh_Size-1
!!          den = (j-1)*A%Mesh_Step
!!          Write(SCRATCH_UNIT, *) j, den, A%Phi(j,i), A%TPhi(j,i), A%TP(j,i), &
!!               A%Core_Density(j), A%TP(j,i)/den, A%Phi(j,i)/den, &
!!               A%TPhi(j,i)/den
!!       End Do
!!       den = den + A%Mesh_Step
       !     Write(SCRATCH_UNIT, *) A%Mesh_Size+1, den, '  0      0     0'
!!       CLose(SCRATCH_UNIT)

!!       Write(token, *) i
!!       token = TRIM(A%Atom_Name) // trim(ADJUSTL(token)) // '.radg'
!!       Open(SCRATCH_UNIT, FILE=token, RECL=10000)
!!
!!       Write(SCRATCH_UNIT, *) '1.Index   2.|G|    3.P(|G|)'
!!       L = A%L_Value(i)
!!       Do j=1, 40
!!          gmag = 0.25*(j-1)
!!          den = RadialFourier(L, Gmag, A%Mesh_Step, A%Mesh_Size, A%TP(:,i))
!!
!!          Write(SCRATCH_UNIT, *) j, gmag, den
!!       End Do
!!       Close(SCRATCH_UNIT)

!!    End Do
  End If

!**NOPROJ**
!*A%Basis_Size = 0
 
  Return
End Subroutine

!**********************************************************************
!
!  LdAtomtype_NORM - Loads an species definition for a norm conserving
!            pseuodpotential definition.
!
!     NOTE : The program must be in SIM_NORM mode!
!
!**********************************************************************

Subroutine LdAtomType_NORM(WC)
  Type (Word_Context), Intent(INOUT) :: WC   

  Character*100 :: Token
  Integer :: tlen, i, j
  Logical :: NewToken
  Real, pointer  :: rtmp(:)
  Type(Atom_Info_Fixed), Pointer :: A

  !** Make sure we are in the correct mode ***
  If (Atomic_Mode == SIM_PAW) then
     Write(Log_Unit,*) 'LdAtomtype_NORM: Attempting to load a norm', &
          ' conserving pseudopotential when in PAW mode!!!!!'
     Call Word_GetAndPrint(WC, Error_Unit, 'ldAtomType_NORM: ')
     STOP
  End If

  if (Atom_types >= Max_Atom_Types) then
     write(Error_Unit, *) 'ldatomtype_norm: Error Loading atom type!'
     write(Error_Unit, *) 'ldatomtype_norm: Not Enough space allocated!'
     write(Error_Unit, *) 'ldatomtype_norm: Max_Atom_Types = ', Max_Atom_Types

     Call Word_GetAndPrint(WC, Error_Unit, 'ldatomtype: ')
     STOP
  End If
  
  !** Find position to store the PP **
  Atom_types = Atom_Types + 1
  A => AtomType_Info(Atom_types)


  Write(Log_Unit,*) 'LdAtomType_NORM: TypeIndex = ', Atom_Types

  !*** Now load the data **
  Call GetNextWord(WC, A%Atom_Name, Tlen)

  If (Print_Level <= PRINT_COMMANDS) then
     Write(Log_Unit, *) ">AtomType ", Trim(A%Atom_Name)
  End IF

  i = SearchAddList(A%Atom_Name, AtomType_LUT, NewToken)
  if (.NOT. NewToken) then     !** Name previously used
     Write(Error_Unit, *) "LdAtomType: Name previously used! Name =", &
                           Trim(A%Atom_Name), " * Index =", i
     Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomType:')
     STOP
  End If
   

  Call GetNextWord(WC, token, Tlen)
  Call UpperCase(Token)

  Do While (Trim(token) /= "END")
    NewToken = .TRUE.

    if (Trim(token) == "BASIS_SIZE") then
       Call GetNumbers(WC, A%Basis_Size)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Basis_Size = ", A%Basis_Size
       End IF
    else if (Trim(token) == "MESH_SIZE") then
       Call GetNumbers(WC, A%Mesh_Size) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Mesh_Size = ", A%Mesh_Size
       End IF
    else if (Trim(token) == "MESH_STEP") then
       Call GetNumbers(WC, A%Mesh_Step) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Mesh_Step = ", A%Mesh_Step
       End IF
    else if (Trim(token) == "ICORR") then
       Call GetNextWord(WC, A%icorr, tlen) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    ICORR = ", TRIM(A%ICORR)
       End IF
    else if (Trim(token) == "IREL") then
       Call GetNextWord(WC, A%irel, tlen) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    IREL = ", TRIM(A%IREL)
       End IF
    else if (Trim(token) == "NICORE") then
       Call GetNextWord(WC, A%nicore, tlen) 
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    NICORR = ", TRIM(A%NICORE)
       End IF
    else if (Trim(token) == "ATOMIC_CHARGE") then
       Call GetNumbers(WC, A%Atomic_Charge)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Atomic_Charge = ", &
               A%Atomic_Charge
       End IF
    else if (Trim(token) == "ALPHA") then
       Call GetNumbers(WC, A%Alpha)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Alpha = ", A%Alpha
       End IF
    else if (Trim(token) == "BASIS_SIZE") then
       Call GetNumbers(WC, A%Basis_Size)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Basis_Size = ", A%Basis_Size
       End IF

       !** Allocate arrays also.  Assumes A%Mesh_Size has been declared! **
       i = A%Basis_Size; j = A%Mesh_Size
       Allocate(A%TP(j+1,i), A%pot_local(j), A%Core_Density(j), &
            A%Density(j))
    else if (Trim(token) == "VALENCE_DENSITY") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Loading Density"
       End IF                
       Call LoadArray(WC, A%Density, A%Mesh_Size, 'ldatomtype_norm: (Valence)')
    else if (Trim(token) == "VALENCE_DENSITY") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Loading Density"
       End IF                
       Call FillArray(WC, A%Density, A%Mesh_Size, 'ldatomtype_norm: (Valence)')
    else if (Trim(token) == "CORE_DENSITY") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Loading Core Density"
       End IF                
       Call FillArray(WC, A%Core_Density, A%Mesh_Size, &
            'ldatomtype_norm: (core)')
    else if (Trim(token) == "V_LOCAL") then
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Loading V_Local"
       End IF                
       Call FillArray(WC, A%Pot_local, A%Mesh_Size, &
            'ldatomtype_norm: (core)')
    else if (Trim(token) == "PROJECTOR") then
       Call GetNumber(WC, i)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype_norm:    Loading Projector", i
       End IF                
       Call FillArray(WC, A%TP(:,i), A%Mesh_Size+1, 'ldatomtype_norm: (proj)')
    else if (Trim(token) == "ORBITALS") then
       Allocate(A%L_Value(A%Basis_Size), STAT=j)
       Call Check_Error(j, 'ldatomtype_norm(orbitals data) Error during allocation!',  &
                Error_Unit, .TRUE., WC, "ldatomtype_norm(orbitals data): ")
       Call LoadArray(WC, RTmp, A%Basis_Size, "orbitals")
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatomtyep_norm(orbitals data): ");     
       A%L_Value = RTmp
       DeAllocate(Rtmp)
       if (Print_Level == PRINT_VERBOSE) then
          Write(Log_Unit, *) "ldatomtype:    Orbitals = ", A%L_Value
       End IF                
    else 
       Write(Error_Unit, *) "ldAtomType_NORM: Unknown Option: ", Trim(Token)
    End If

    If (NewToken) then
       Call GetNextWord(WC, token, tlen)
       Call UpperCase(Token)
       if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatomtype_norm(next word): ")
    End If
  End Do
    
  Return
End Subroutine

!**********************************************************************
!
!  LdAtomtype_SEPM - Loads a species definition for a semi-empirical
!            pseuodpotential definition.
!
!     NOTE : The program must be in SIM_SEPM mode!
!
!**********************************************************************

Subroutine LdAtomType_SEPM(WC)
  Type (Word_Context), Intent(INOUT) :: WC   

  Character*100 :: token, token2
  Character*100 :: atomname(2)
  Real          :: Gaussian(2,2), Poly_Gcut, alpha, beta, x, dx, fn, tmp
  Type (GaussFunc) :: VIn(2), VOut(2)
!*  Real          :: PolyIn(5,2), PolyOut(5,2)
  Integer       :: i, tlen, Poly_Size(2), index, sign, n, fd
  Logical       :: Newtoken
  Type(Atom_Info_Fixed), Pointer :: A

  !** Make sure we are in the correct mode ***
  If (Atomic_Mode /= SIM_SEPM) then
     Write(Log_Unit,*) 'LdAtomtype_SEPM: Attempting to load a semi', &
          '-empirical pseudopotential when not in SEPM mode!!!!!'
     Call Word_GetAndPrint(WC, Error_Unit, 'ldAtomType_SEPM: ')
     STOP
  End If

  if (Atom_types >= Max_Atom_Types) then
     write(Error_Unit, *) 'ldatomtype_SEPM: Error Loading atom type!'
     write(Error_Unit, *) 'ldatomtype_SEPM: Not Enough space allocated!'
     write(Error_Unit, *) 'ldatomtype_SEPM: Max_Atom_Types = ', Max_Atom_Types

     Call Word_GetAndPrint(WC, Error_Unit, 'ldatomtype_SEPM: ')
     STOP
  End If
  
  !** Find position to store the PP **
  Atom_types = Atom_Types + 1
  A => AtomType_Info(Atom_types)


  Write(Log_Unit,*) 'LdAtomType_NORM: TypeIndex = ', Atom_Types

  !*** Now load the data **
  Call GetNextWord(WC, A%Atom_Name, Tlen)

  If (Print_Level <= PRINT_COMMANDS) then
     Write(Log_Unit, *) ">AtomType ", Trim(A%Atom_Name)
  End IF

  i = SearchAddList(A%Atom_Name, AtomType_LUT, NewToken)
  if (.NOT. NewToken) then     !** Name previously used
     Write(Error_Unit, *) "LdAtomType_SEPM: Name previously used! Name =", &
                           Trim(A%Atom_Name), " * Index =", i
     Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomType_SEPM:')
     STOP
  End If

  Call GetNextWord(WC, token, tlen)
  Call UpperCase(token)

  Do While ((W_Error /= W_EOF) .AND. (Trim(Token) /= 'END'))

Write(*,*) 'Read_Input : Token: !',trim(token),'!'
    If (Trim(Token) == "GAUSSIAN") then
       Call GetNumbers(WC, A%Gaussian)
    else if (Trim(Token) == "POTENTIAL") then
       Call GetNumbers(WC, i)
       A%V_sepm%N = i
       Allocate(A%V_sepm%Coeff(3,i))
       Do i=1, A%V_sepm%N
          Call GetNumbers(WC, A%V_sepm%Coeff(:,i))
       End do
       Call GetNExtword(WC, token, tlen)
    else if (Trim(Token) == "POLY_GCUT") then
       Call GetNumbers(WC, Poly_Gcut)
    else if (Trim(Token) == "POLY") then
       Call GetNumbers(WC, i)
       A%Poly_Degree = i
       Allocate(A%Poly(i+1))
       Call GetNumbers(WC, A%Poly)
       Call GetNExtword(WC, token, tlen)
    else
       Write(*, *) "ldatomtype_SEPM: Unknown Command : ", Trim(Token)
       Call Word_GetAndPrint(WC, Error_Unit, 'ldatomtype_SEPM:')
    End If    

    Call GetNextWord(WC, Token, tlen)
    Call UpperCase(token)
  end do

  fd = SCRATCH_UNIT
  token = trim(A%Atom_Name) // '.txt'
  Open(fd, FILE=trim(token), RECL=10024)
  Write(fd,*) '1.x 2.V_sepm'

  n = 200
  dx = 10.0/n
  Poly_Gcut = A%Poly_Gcut
Poly_Gcut = -1
  do i=0, n
     x = i*dx
     If (x >= Poly_Gcut) then
        fn = EvalGauss(A%V_sepm, x)
     else
        fn = EvalPoly(A%Poly_Degree, A%Poly, x)
     End If

     Write(fd,*) x, fn
  End Do
  Close(fd)

  Return
End Subroutine

!******************************************************************************
!
!  Clone_Cell - Clones the current supercell definition and modifies the
!               lattice to reflect the larger cell.
!
!     Cell - Clone size for each principal axis = 0 0 0 if no cloning
!
!******************************************************************************

Subroutine Clone_Cell(Cell)
  Integer, Intent(IN) :: Cell(:)

  Type (Specific_Atom), Pointer :: A, BA
  Integer :: i,j,k,  Base_Atoms, index, ctmp(3),s,t,MaxL,err
  Real    :: Offset(3), OldBasis(3,3), OldInversebasis(3,3),tol,rs,rt
  character*20 :: Name_Ext, strtmp
  logical :: IsNew

  tol = 1E-8
  OldBasis=Xtal%Basis
  OldInversebasis=Xtal%Inversebasis
     !** First Adjust the lattice **

     Do i=1, 3
        xtal%Basis(:,i) = OldBasis(:,i)*(Cell(i)+1)
     End do

     xtal%Volume = ABS(DOT_PRODUCT(xtal%Basis(:,1), &
          CrossProduct(xtal%Basis(:,2), xtal%Basis(:,3))))
     xtal%Recip = RecipBasis(xtal%Basis)
     xtal%InverseBasis = TRANSPOSE(Xtal%Recip) / (2*Pi)
     xtal%RecipVol = ABS(DOT_PRODUCT(xtal%Recip(:,1), &
          CrossProduct(xtal%Recip(:,2), xtal%Recip(:,3))))

     If (Print_Level <= PRINT_NORMAL) then
        Write(Log_Unit,*) 'Clone_Cell: New Xtal information. Cell=',Cell
        Write(Log_Unit, *) 'Clone_Cell: Xtal Volume : ', Xtal%Volume, &
             ' * Recip Volume :',Xtal%RecipVol
        Write(Log_Unit, *) 'Clone_Cell: Scaled Lattice Vectors:' 
        Write(Log_Unit, *) 'Clone_Cell: A :',Xtal%Basis(:,1)
        Write(Log_Unit, *) 'Clone_Cell: B :',Xtal%Basis(:,2)
        Write(Log_Unit, *) 'Clone_Cell: C :',Xtal%basis(:,3)
        Write(Log_Unit,*)  ' '
        Write(OutPut_Unit, *) ' Xtal Volume : ', Xtal%Volume
        Write(OutPut_Unit, *) 'Lattice Vectors:' 
        Write(OutPut_Unit, *) ' A :',Xtal%Basis(:,1)
        Write(OutPut_Unit, *) ' B :',Xtal%Basis(:,2)
        Write(OutPut_Unit, *) ' C :',Xtal%basis(:,3)
        Write(OutPut_Unit,*)  ' '
        Write(Log_Unit, *) 'Clone_Cell: Recip(:,1) :',Xtal%Recip(:,1)
        Write(Log_Unit, *) 'Clone_Cell: Recip(:,2) :',Xtal%Recip(:,2)
        Write(Log_Unit, *) 'Clone_Cell: Recip(:,3) :',Xtal%Recip(:,3)
     end If
 !********* Scale the BASE atom positions and init them ***************
  Do i=1, Specific_Atoms
     rt = sum(abs(Atom_List(i)%Frac_Pos))
     rs = sum(abs(Atom_List(i)%Pos))
!Write(Log_Unit,*) 'Clone_Cell: i=',i, ' * t,s=',rt,rs
!Write(Log_Unit,*) 'Clone_Cell: Frac=',Atom_List(i)%Frac_Pos
!Write(Log_Unit,*) 'Clone_Cell:  Pos=',Atom_List(i)%Pos
     if ((rs<=tol).and.(rt>tol)) then
         Atom_List(i)%Pos = MATMUL(OldBasis, Atom_List(i)%Frac_Pos)
      else if ((rt<=tol).and.(rs>tol)) then
         Atom_List(i)%Frac_Pos = &
              MATMUL(OldInversebasis, Atom_List(i)%Pos)
      else if ((rt>tol).and.(rs>tol)) then
         write(Log_Unit,*)'Warning -- both fractional and Cartesian ', &
           'positions specified in atom file'
         Atom_List(i)%Pos = MATMUL(OldBasis, Atom_List(i)%Frac_Pos)
      else
         Atom_List(i)%Pos = 0 ; Atom_List(i)%Frac_Pos = 0
      endif

     Write(Log_Unit,*) 'Clone_Cell: atom=',i, ' * ',&
          Trim(Atom_List(i)%Name)
     Write(Log_Unit,*) 'Clone_Cell:   Pos=',Atom_List(i)%Pos
     Write(Log_Unit,*) 'Clone_Cell:   Frac_Pos=',Atom_List(i)%Frac_Pos
  End Do

     Base_Atoms = Specific_Atoms
     Specific_Atoms=0
     Do i=0, Cell(1)
        Do j=0, Cell(2)
           Do k=0, Cell(3)              
              ctmp = (/i,j,k/)
              Offset = MATMUL(OldBasis, (/i,j,k/))

              !** Now loop over all the base atoms **
              Do index=1, Base_Atoms
                 Specific_Atoms = Specific_Atoms + 1

                 if (Specific_Atoms > Max_Specific_Atoms) then
                    write(Error_Unit, *) &
                     'Clone_Cell: Error Loading a specific atom!'
                    write(Error_Unit, *) &
                     'Clone_Cell: Not Enough space allocated!'
                    write(Error_Unit, *) &
                     'Clone_Cell: Max_Specific_Atoms = ', Max_Specific_Atoms
                    
                    STOP
                 End If

                 A => Atom_list(Specific_Atoms)
                 BA => Atom_List(Index)


              If (i+j+k > 0) then
                 !** Construct the name and add it to the LUT **
                 Name_Ext = '_'
                 Do t=1, 3
                    Write(strtmp, *) ctmp(t)
                    Name_Ext = TRIM(Name_Ext) // TRIM(ADJUSTL(strtmp))
                 end Do
                 A%Name = Trim(BA%Name) // TRIM(Name_Ext)

                 s = SearchAddList(A%Name, Atom_LUT, IsNew)
                 if (.NOT. IsNew) then     !** Name previously used
                    Write(Error_Unit, *) &
                      "LdAtomList: Name previously used! Name =", &
                         Trim(A%Name), " * Index =", i
                    Call Word_GetAndPrint(paw_WC, Error_Unit,'Clone_Cell:')
                    STOP
                 End If

                 !** Copy the Base data ***
                 s = BA%Orbitals_Size
                 A%Orbitals_Size = BA%Orbitals_Size
                 Allocate(A%Valence_Orbitals(s), A%Init_Occ(s))
                 A%Valence_Orbitals = BA%Valence_Orbitals
                 A%Init_Occ = BA%Init_Occ
                 A%Freeze = BA%Freeze
                 A%Friction = BA%Friction
                 A%Enhance_Size = BA%Enhance_Size
                 A%TypeIndex = BA%TypeIndex


                 If (Atomic_Mode == SIM_PAW) then
                    MaxL = MaxVal(AtomType_Info(A%TypeIndex)%L_Value)
                    s = AtomType_Info(A%TypeIndex)%nlm_Size
                    t = s*(s+1)/2              !**  Determine unique Dij's

                    Allocate(A%Qlm(2*MaxL+1, 2*(2*MaxL)+1), &
                         A%dEdQlm(2*MaxL+1,2*(2*MaxL)+1), A%Dij(s,s), &
                         A%Dij_old(t),  A%Wij(s,s), A%Wij_old(s,s), &
                         A%Fij(3,s,s), A%Feij(3,s,s), STAT=err)

                    A%Qlm = 0;  A%dEdQlm = 0;  A%Dij = 0;  A%Wij = 0;
                    A%Wij_old = 0;  A%Dij_old = 0; A%Fij = 0;  A%Feij = 0

                    Call Check_Error(err, &
                         "Clone_Cell: Error allocating Qlm, ...  ",  &
                         Error_Unit, .TRUE., paw_wc, "ldatom:")
                 EndIf
              EndIf

                 A%Pos = BA%Pos + Offset  !** Update the position
                 A%Frac_Pos = MATMUL(Xtal%Inversebasis, A%Pos)

                 !** Init the rest of the structure
                  A%Force%Pos(:,1) = A%Pos
                  A%Force%Velocity = 0
                  A%Force%Acceleration = 0
                  A%Force%NumMoves = 1
                  A%Force%Predictor_Phase = .TRUE.

                 Write(Log_Unit,*) 'Clone_Cell: atom=',Specific_Atoms, ' * ',&
                      Trim(A%Name)
                 Write(Log_Unit,*) 'Clone_Cell:   Pos=',A%Pos
                 Write(Log_Unit,*) 'Clone_Cell:   Frac_Pos=', &
                      A%Frac_Pos
              end do
           End Do
        End Do
     End do

  return
end subroutine

!******************************************************************************
!
!  ldAtom - Loads a single atom from the input file.
!           The input format is specified below:
!
!   MAXATOM  Max_Specific_Atom     # Previously defined
!
!   ATOM  AtomName
!     TYPE AtomType                 # Type of atom corresponds to an ATOMTYPE
!     FRAC_POSITION (fa,fb,fc)      # Initial  Position of atom 
!    /                                   (in fractions of A, B, and C)
! or
!    \
!     CART_POSITION (fa,fb,fc)      # Initial Position of atom 
!                                      (in Cartesian coordinates -- bohr units)
!     ORBITALS_SIZE n               # Number of valence orbitals for LCAO
!     VALENCE_ORBITALS i1 i2 .. in  # List of (nl)'s for use as valence fn's
!     VALENCE_OCCUPANCY q1 q2 .. qn # Initial charge distribution in electrons.
!     FRICTION fc                   # Friction coeff (OPTIONAL)
!
!     Enhance N            # Number of enhance definitions
!       Level_1  Radius_1      # Radius is in Bohr
!       Level_2  Radius_2
!         ...       ...
!       Level_N  Radius_N
!     End 
!   END
!
!******************************************************************************

Subroutine ldAtom(WC)
  Type (Word_Context), Intent(INOUT) :: WC

  Type (Specific_Atom), Pointer :: A
  Integer                       :: tlen, i, j, k, n, err, MaxL
  character*100                 :: token
  Logical                       :: IsNew

  if (Specific_Atoms >= Max_Specific_Atoms) then
     write(Error_Unit, *) 'ldatom: Error Loading a specific atom!'
     write(Error_Unit, *) 'ldatom: Not Enough space allocated!'
     write(Error_Unit, *) 'ldatom: Max_Specific_Atoms = ', Max_Specific_Atoms

     Call Word_GetAndPrint(WC, Error_Unit, 'ldatom: ')
     STOP
  End If

  Specific_Atoms = Specific_Atoms + 1
  A => Atom_list(Specific_Atoms)
 
  A%Frac_Pos = 0; A%Pos = 0;   A%TypeIndex = -1; A%Orbitals_Size = -1;
  A%Freeze = .FALSE.;   A%Friction = 0;  A%Enhance_Size = -1

  Call GetNextWord(WC, A%Name, tlen)

  If (Print_Level <= PRINT_COMMANDS) then
     Write(Log_Unit, *) ">Atom ", Trim(A%Name)," * Array index=",Specific_Atoms
  End IF

  i = SearchAddList(A%Name, Atom_LUT, IsNew)
  if (.NOT. IsNew) then     !** Name previously used
     Write(Error_Unit, *) "LdAtom: Name previously used! Name =", &
                           Trim(A%Name), " * Index =", i
     Call Word_GetAndPrint(WC, Error_Unit, 'LdAtom:')
     STOP
  End If

  Call GetNextWord(WC, token, Tlen)
  Call UpperCase(Token)
  
  Do While (Trim(token) /= "END")
    if (Trim(token) == "TYPE") then
       Call GetNextWord(WC, token, tlen)
       i = SearchList(token, AtomType_LUT)
 
       if (i <= 0) then     !** No MAtch Found!
          Write(Error_Unit,*) 'LdAtom: Unknown Atom Type! Type = ', Trim(token)
          Write(Error_Unit,*) 'LdAtom: Current Atom Types defined: Used=', &
               AtomType_LUT%Used
          Write(Error_Unit, *) 'LdAtom: Names = ', &
               (Trim(AtomType_LUT%StrList(i)), ' * ', i=1, Atomtype_LUT%USed)
          Call Word_GetAndPRint(WC, Error_Unit, "LdAtom:")
          STOP
       else
         A%TypeIndex = i

         If (Print_Level <= PRINT_COMMANDS) then
            Write(Log_Unit, *) "ldatom:  AtomType =", &
                 TRim(AtomType_Info(i)%Atom_Name)
         End IF
      End IF
    else if (Trim(Token) == "FRAC_POSITION") then
       Call GetNumbers(WC, A%Frac_Pos, 3)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Fractional Position = ", A%Frac_Pos
       End IF 
    else if (Trim(Token) == "CART_POSITION") then
       Call GetNumbers(WC, A%Pos, 3)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Cartesian Position = ", A%Pos
       End IF 
    else if (Trim(token) == "ORBITALS_SIZE") then
       Call GetNumber(WC, i)
       A%Orbitals_Size = i

       Allocate(A%Valence_Orbitals(i), STAT=j)
       Write(token,*) "ldatom: Can't alloc space for Valence Orbitals! Size=",i
       Call Check_Error(j, token, Error_Unit, .TRUE., WC, "ldatom:")

       Allocate(A%Init_Occ(i), STAT=j)
       Write(token,*)"ldatom: Can't alloc space for Valence Occupancy! Size=",i
       Call Check_Error(j, token, Error_Unit, .TRUE., WC, "ldatom:")
    else if (Trim(token) == "VALENCE_ORBITALS") then
       If (A%Orbitals_Size <= 0) then
          Write(Error_Unit, *) 'ldatom: Orbital_Size not defined!!'
          Call Word_GetAndPrint(WC, Error_Unit, "ldatom: ")
          STOP
       End If

       Call GetNumbers(WC, A%Valence_Orbitals)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Valence_Orbitals = ", A%Valence_Orbitals
       End IF 
    else if (TRim(token) == "VALENCE_OCCUPANCY") then
       If (A%Orbitals_Size <= 0) then
          Write(Error_Unit, *) 'ldatom: Orbital_Size not defined!!'
          Call Word_GetAndPrint(WC, Error_Unit, "ldatom: ")
          STOP
       End If

       Call GetNumbers(WC, A%Init_Occ)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Valence_Occupancy = ", A%Init_Occ
       End IF 
    else if (Trim(Token) == "FRICTION") then
       Call GetNumber(WC, A%Friction)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Friction = ", A%Friction
       End IF 
    else if (Trim(token) == "ENHANCE") then
       Call GetNumber(WC, A%Enhance_Size)
       i = A%Enhance_Size
       Allocate(A%enhance(i), STAT=j)
       Write(token,*) "ldatom: Can't alloc space for enhancements! Size=",i
       Call Check_Error(j, token, Error_Unit, .TRUE., WC, "ldatom:")

       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldatom:  Enhance_Size = ", A%Enhance_Size
       End IF        
       Do i=1, A%Enhance_Size
          Call GetNumber(WC, A%Enhance(i)%Level)
          Call GetNumber(WC, A%Enhance(i)%Radius)
          if (Print_Level == PRINT_VERBOSE) then
             write(LOG_Unit, *) "ldatom:    Enhance(L,R):", &
                  A%Enhance(i)%Level, A%Enhance(i)%Radius
          End IF
       End Do

       Call GetNextWord(WC, token, tlen)
    else
       Write(Error_Unit, *) "ldAtom: Unknown Option: ", Trim(Token)
    End If

    Call GetNextWord(WC, token, tlen)
    if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldatom(next word): ")
    Call UpperCase(Token)
  End Do       

  A%Nuclear_Params = 0
  A%Force%NumMoves = 0;  A%Force%Predictor_Phase = .FALSE.

  If (Atomic_Mode == SIM_PAW) then
     MaxL = MaxVal(AtomType_Info(A%TypeIndex)%L_Value)
     i = AtomType_Info(A%TypeIndex)%nlm_Size
     n = i*(i+1)/2              !**  Determine unique Dij's

     Allocate(A%Qlm(2*MaxL+1, 2*(2*MaxL)+1), A%dEdQlm(2*MaxL+1,2*(2*MaxL)+1), &
          A%Dij(i,i),  A%Dij_old(n), A%Wij(i,i), A%Wij_old(i,i), &
          A%Fij(3,i,i),  A%Feij(3,i,i), STAT=err)

     A%Qlm = 0;  A%dEdQlm = 0;  A%Dij = 0;  A%Wij = 0; A%Wij_old = 0
     A%Fij = 0;  A%Feij   = 0;  A%Dij_old = 0
  End If

  Call Check_Error(err, "LdAtom: Error allocating Qlm, ...",  &
       Error_Unit, .TRUE., WC, "ldatom:")  

  Return
End Subroutine


!******************************************************************************
!
!  ldAtomList - Loads a list of atoms from the input file.
!           The input format is specified below:
!
!   MAXATOM  Max_Specific_Atom     # Previously defined
!
!   ATOM_LIST [CART_POSITION | FRAC_POSITION]
!       Name1 Type1 (x1,y1,z1)
!       Name2 Type2 (x2,y2,z2)
!        ..    ..       ..
!       NameN TypeN (xN,yN,zN)
!   END
!
!
!******************************************************************************

Subroutine ldAtomList(WC)
  Type (Word_Context), Intent(INOUT) :: WC

  Character*100 :: token, token2
  Integer :: i, n, tlen, Mode, MaxL, err
  Real    :: pos(3),scale
  logical :: IsNew
  Type(Atom_Info_Fixed), Pointer :: AT
  Type (Specific_Atom), Pointer :: A
  
  !*** Determine coordinate format ***
  Call GetNextWord(WC, token, Tlen)
  Call UpperCase(Token)  
  If (TRIM(token) == "FRAC_POSITION") then
     mode = 1
  else if (TRIM(token) == "CART_POSITION") then
     mode = 2
  else 
     Write(Error_Unit, *) 'ldAtomList: Unknown Mode:',trim(token)
     Write(Error_Unit, *) 'ldAtomList: Assuming FRAC_POSITION mode.'
  End If

  Write(Log_Unit,*) 'LdAtomList: Mode=',TRIM(Token)
  Scale = 1


  Call GetNextWord(WC, token, Tlen)
  token2 = token
  Call UpperCase(Token2)  

     If(Token2=="SCALE") then
        Call GetNumber(WC,Scale)
             If (Scale<1.e-8) Scale = 1
             Call GetNextWord(WC, token, Tlen)
             token2 = token
             Call UpperCase(Token2)  
     EndIf
       
  Do While (TRIM(Token2) /= "END")
     !**** Inc slot for new atom ***
     Specific_Atoms = Specific_Atoms + 1
     A => Atom_list(Specific_Atoms)
 
     A%Name = token
     A%Frac_Pos = 0; A%Pos = 0;   A%TypeIndex = -1; A%Orbitals_Size = -1;
     A%Freeze = .FALSE.;   A%Friction = 0;  A%Enhance_Size = -1

     i = SearchAddList(A%Name, Atom_LUT, IsNew)
     if (.NOT. IsNew) then     !** Name previously used
        Write(Error_Unit, *) "LdAtomList: Name previously used! Name =", &
             Trim(A%Name), " * Index =", i
        Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomList:')
        STOP
     End If

     !** Determine the type index ***
     Call GetNextWord(WC, token, Tlen)
     i = SearchList(token, Atomtype_LUT)
     If (i<0) then  !** type not found so ABORT 
        Write(Error_Unit, *) "LdAtomList: type Not Found! Type =", Trim(Token)
        Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomList:')
     End if

     A%TypeIndex = i
     AT => AtomType_Info(i)

     A%Orbitals_size = AT%Orbitals_Size
     i = A%Orbitals_size
     Allocate(A%Valence_Orbitals(i), A%Init_Occ(i))
     A%Valence_orbitals = AT%Valence_Orbitals
     A%Init_Occ = AT%Init_Occ

     !*** Now get the position and store it in the appropriate slot ***
     Call GetNumbers(WC, Pos, 3)
     If (Mode == 1) then
        A%Frac_Pos = Pos*Scale
     else 
        A%Pos = Pos*Scale
     End If

     !*** Write a synopsis line ***
     Write(Log_Unit,*) TRIM(A%Name), ' ', Trim(AT%Atom_Name), Pos


     !*** Finally init the rest of the data structure ***
     A%Nuclear_Params = 0
     A%Force%NumMoves = 0;  A%Force%Predictor_Phase = .FALSE.
     
     If (Atomic_Mode == SIM_PAW) then
        MaxL = MaxVal(AtomType_Info(A%TypeIndex)%L_Value)
        i = AtomType_Info(A%TypeIndex)%nlm_Size
        n = i*(i+1)/2              !**  Determine unique Dij's
        
        Allocate(A%Qlm(2*MaxL+1, 2*(2*MaxL)+1), &
             A%dEdQlm(2*MaxL+1,2*(2*MaxL)+1), A%Dij(i,i), &
             A%Dij_old(n),  A%Wij(i,i), A%Wij_old(i,i), &
             A%Fij(3,i,i), A%Feij(3,i,i), STAT=err)
        
        A%Qlm = 0;  A%dEdQlm = 0;  A%Dij = 0;  A%Wij = 0; A%Wij_old = 0
        A%Dij_old = 0; A%Fij = 0;  A%Feij   = 0

        Call Check_Error(err, "LdAtomList: Error allocating Qlm, ...",  &
             Error_Unit, .TRUE., WC, "ldatom:")  
     End If


     !*** check for another line ***
     Call GetNextWord(WC, token, Tlen)
     token2 = token
     Call UpperCase(Token2)  
  End Do

  Return
End Subroutine

!******************************************************************************
! ldShellInfo -- Sets up Shell parameters and translation to atomic positions
!
!******************************************************************************

Subroutine ldShellInfo(WC,WithOrigin)
  Type (Word_Context), Intent(INOUT) :: WC
  Logical, Intent(IN)                :: WithOrigin

  Character*100 :: token, token2, token3
  Integer :: tlen, i,a, j, k, index, ier
  Real    :: vec(3),scale
  
 If (Specific_Atoms<1) then
  Write(Error_Unit,*) 'Error in ldShellInfo - Specific_Atoms<1',Specific_Atoms
  Write(Error_Unit,*) 'Must read in atomic positions first'
  stop
 EndIf

 Call GetNumber(WC,i)
   If (i>0) then
      Shell%NPARAMS=i
      Write(Log_Unit,*)'ldShellInfo: Shell%NPARAMS',i
     Else
      Write(Error_Unit,*) 'Error in ldShellInfo -- NPARAMS = ',i
      Stop
   EndIf

 Allocate(Shell%C(i),Shell%ShellForce(i),stat=ier)
   If (ier > 0 ) then
      Write(Error_Unit,*) 'Error in ldShellInfo allocation',ier
      stop
   EndIf

 Nullify(Shell%AtomMap)
 Allocate(Shell%AtomMap(Specific_Atoms),stat=ier)
   If (ier > 0 ) then
      Write(Error_Unit,*) 'Error in ldShellInfo allocation',ier
      stop
   EndIf

   Do i=1,Specific_Atoms
      Shell%AtomMap(i)%Origin = 0
   Enddo

  Call GetNextWord(WC, token, Tlen)
  token2 = token
  Call UpperCase(Token2)  

! List of atoms mapped to parameters
  Do While (TRIM(Token2) /= "END")
     a=0
     loop:  do i = 1,Specific_Atoms
        If(TRIM(token)==TRIM(Atom_List(i)%Name)) then
           a=i
           exit loop
        EndIf
     enddo loop
  
     If (a<1) then
        Write(Error_Unit,*) 'Error in ldShellInfo -- atom name', token
        stop
     EndIf
     
     Call GetNumber(WC,i)
       Shell%AtomMap(a)%MapParams=i  !**Can be zero if atom constrained
       Write(Log_Unit,*)'ldShellInfo: a, MapParams',a,i
       If (i > 0 ) then  
         If (i > 3) then
            Write(Error_Unit,*) 'Error in ldShellInfo -- MapParams>3',&
               token,a,i
            stop
         EndIf

         Nullify(Shell%AtomMap(a)%Map); Nullify(Shell%AtomMap(a)%V)
         
         Allocate(Shell%AtomMap(a)%Map(i),Shell%AtomMap(a)%V(3,i),stat=ier)
          If (ier > 0 ) then
             Write(Error_Unit,*) 'Error in ldShellInfo allocation',ier
             stop
          EndIf

         If (WithOrigin) then 
            Call GetNextWord(WC, token3, Tlen)
            Call UpperCase(token3)
              If (TRIM(token3)=="O_CART") then
                 Call GetNumbers(WC,vec,3)
                 Write(Log_Unit,*)'ldShellInfo: origin',vec
                 Shell%AtomMap(a)%Origin = vec
              Elseif (TRIM(token3)=="O_SCALE") then
                 Call GetNumber(WC,scale)
                 Call GetNumbers(WC,vec,3)
                 Write(Log_Unit,*)'ldShellInfo: scaled origin',scale,vec
                 Shell%AtomMap(a)%Origin = vec*scale
              Elseif (TRIM(token3)=="O_FRAC") then
                 Call GetNumbers(WC,vec,3)
                 Write(Log_Unit,*)'ldShellInfo: origin',vec
                 Shell%AtomMap(a)%Origin = MATMUL(xtal%Basis,vec)
              Else
                 Write(Log_unit,*) 'ldShellInfo error in origin', token3
                 stop
              EndIf
         EndIf

         Do j=1,i
           Call GetNumber(WC,k)
            If ((k < 1) .OR. (k > Shell%NPARAMS)) then
              Write(Error_Unit,*) 'Error in ldShellInfo -- Map',a,j,k
              Stop
            EndIf
           Shell%AtomMap(a)%Map(j) = k
           Write(Log_Unit,*)'ldShellInfo: j,Map',j,k
           Call GetNumbers(WC,vec,3)
           Write(Log_Unit,*)'ldShellInfo: vec',vec   
 
            scale=Dot_Product(vec,vec)
             If (scale > 1.e-15) then
                Shell%AtomMap(a)%V(:,j)=vec/SQRT(scale) !Normalize to unity
             Else
                Write(Error_Unit,*) &
                 'Error in ldShell -- vector',a,j,scale
                Stop
             EndIf
         Enddo
       EndIf


     !*** check for another line ***
     Call GetNextWord(WC, token, Tlen)
     token2 = token
     Call UpperCase(Token2)  
  EndDo

  Return
End Subroutine

!******************************************************************************
!
!  ldAtomTypeOcc - Loads the default occupany for the species.
!           The input format is specified below:
!
!   ATOMTYPE_OCCUPANCY Species_Name
!     ORBITALS_SIZE n               # Number of valence orbitals for LCAO
!     VALENCE_ORBITALS i1 i2 .. in  # List of (nl)'s for use as valence fn's
!     VALENCE_OCCUPANCY q1 q2 .. qn # Initial charge distribution in electrons.
!   END
!   
!******************************************************************************

Subroutine ldAtomTypeOcc(WC)
  Type (Word_Context), Intent(INOUT) :: WC

  Character*100 :: token
  Integer :: i, j, tlen
  Type(Atom_Info_Fixed), Pointer :: A

  !*** Get the species name and find its TypeIndex ***
  Call GetNextWord(WC, token, tlen)   

  i = SearchList(token, AtomType_LUT)
  if (i==-1) then     !** Name not found
     Write(Error_Unit, *) "LdAtomTypeOcc: type not found! Name =", TRIM(token)
     Call Word_GetAndPrint(WC, Error_Unit, 'LdAtomTypeOcc:')
     STOP
  End If

  A => AtomType_Info(i)
  
  If (A%Orbitals_Size > 0) then  !** Free old defaults if needed
     A%Orbitals_Size = -1
     DeAllocate(A%Valence_orbitals, A%Init_Occ)
  End if

  Write(Log_Unit,*) 'ldAtomTypeOcc: Atom type:',TRIM(token), ' * Index=',i

  Call GetNextWord(WC, token, Tlen)
  Call UpperCase(Token)
  
  Do While (Trim(token) /= "END")
    if (Trim(token) == "ORBITALS_SIZE") then
       Call GetNumber(WC, i)
       A%Orbitals_Size = i

       Allocate(A%Valence_Orbitals(i), STAT=j)
       Write(token,*) "ldatomTypeOcc: Can't alloc space for Valence Orbitals! Size=",i
       Call Check_Error(j, token, Error_Unit, .TRUE., WC, "ldAtomTypeOcc:")

       Allocate(A%Init_Occ(i), STAT=j)
       Write(token,*)"ldAtomTypeOcc: Can't alloc space for Valence Occupancy! Size=",i
       Call Check_Error(j, token, Error_Unit, .TRUE., WC, "ldAtomTypeOcc:")
    else if (Trim(token) == "VALENCE_ORBITALS") then
       If (A%Orbitals_Size <= 0) then
          Write(Error_Unit, *) 'ldAtomTypeOcc: Orbital_Size not defined!!'
          Call Word_GetAndPrint(WC, Error_Unit, "ldAtomTypeOcc: ")
          STOP
       End If

       Call GetNumbers(WC, A%Valence_Orbitals)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldAtomTypeOcc:  Valence_Orbitals = ", A%Valence_Orbitals
       End IF 
    else if (TRim(token) == "VALENCE_OCCUPANCY") then
       If (A%Orbitals_Size <= 0) then
          Write(Error_Unit, *) 'ldAtomTypeOcc: Orbital_Size not defined!!'
          Call Word_GetAndPrint(WC, Error_Unit, "ldatom: ")
          STOP
       End If

       Call GetNumbers(WC, A%Init_Occ)
       if (Print_Level == PRINT_VERBOSE) then
          write(LOG_Unit, *) "ldAtomTypeOcc:  Valence_Occupancy = ", A%Init_Occ
       End IF 
    else if (TRim(token) == "RS_SCALE") then
       Call GetNumbers(WC, A%Rc_RS_Scale)
       Write(Log_Unit,*) "ldAtomTypeOcc:  RS_Scale = ",A%Rc_Rs_Scale
    else if (TRim(token) == "GCUT") then
       Call GetNumbers(WC, A%Gcut_Proj)
       Write(Log_Unit,*) "ldAtomTypeOcc:  Gcut = ",A%Gcut_PROJ
    else
       Write(Error_Unit, *) "ldAtomTypeOcc: Unknown Option: ", Trim(Token)
    End If

    Call GetNextWord(WC, token, tlen)
    if (W_ERROR /= W_OK) Call EOF_Error(WC, "ldAtomTypeOcc(next word): ")
    Call UpperCase(Token)
  End Do

  Return
End Subroutine


End Module
